searchNodes=[{"doc":"TOML for Elixir This is a TOML library for Elixir projects. It is compliant with version 1.0 of the official TOML specification . You can find a brief overview of the feature set below, but you are encouraged to read the full spec at the link above (it is short and easy to read!). Features Decode from string, file, or stream Fully compliant with the latest version of the TOML spec Is tested against toml-test , a test suite for spec-compliant TOML encoders/decoders, used by implementations in multiple languages. The test suite has been integrated into this project to be run under Mix so that we get better error information and so it can run as part of the test suite. Decoder produces a map with values using the appropriate Elixir data types for representation Supports extension via value transformers (see Toml.Transform docs for details) Supports use as a configuration provider in Distillery 2.x+ (use TOML files for configuration!) Decoder is written by hand to take advantage of various optimizations. Library passes Dialyzer checks Comparison To Other Libraries I compared toml to four other libraries: toml_elixir tomlex jerry etoml Of these four, none correctly implement the 0.5.0 specification. Either they are targeting older versions of the spec (in etoml , it is built against pre-0.1), are not fully implemented (i.e. don't support all features), or have bugs which prevent them from properly parsing a 0.5.0 example file (the test/fixtures/example.toml file in this repository). If you are looking for a TOML library, at present toml is the only one which full implements the spec and correctly decodes example.toml . Installation This library is available on Hex as :toml , and can be added to your deps like so: def deps do [ { :toml , &quot;~&gt; 0.7&quot; } ] end NOTE: You can determine the latest version on Hex by running mix hex.info toml . Type Conversions In case you are curious how TOML types are translated to Elixir types, the following table provides the conversions. NOTE: The various possible representations of each type, such as hex/octal/binary integers, quoted/literal strings, etc., are considered to be the same base type (e.g. integer and string respectively in the examples given). TOML Elixir String String.t (binary) Integer integer inf :infinity +inf :infinity -inf :negative_infinity nan :nan +nan :nan -nan :negative_nan Boolean boolean Offset Date-Time DateTime.t Local Date-Time NaiveDateTime.t Local Date Date.t Local Time Time.t Array list Table map Table Array list(map) Implementation-specific Behaviors Certain features of TOML have implementation-specific behavior: -inf , inf , and +inf are all valid infinity values in TOML. In Erlang/Elixir, these don't have exact representations. Instead, by convention, :infinity is used for positive infinity, as atoms are always larger than integers when using comparison operators, so :infinity &gt; &lt;any integer&gt; will always be true. However, negative infinity cannot be represented, as numbers are always considered smaller than every other type in term comparisons. Instead, we represent it with :negative_infinity , so that the type information is not lost, but you must be careful to deal with it specifically in comparisons/sorting/etc. -nan , nan , and +nan are all valid NaN (not a number) values in TOML. In Erlang/Elixir, NaN is traditionally represented with :nan , but there is no representation for negative NaN, and no API actually produces :nan , instead invalid numbers typically raise errors, in the typical spirit of &quot;let it crash&quot; in the face of errors. For purposes of preserving type information though, we use the :nan convention, and :negative_nan for -NaN. You will need to take care to deal with these values manually if the values need to be preserved. The maximum precision of times in the various time types is microseconds (i.e. precision to six decimal places), if you provide higher precision values (i.e. nanoseconds), the extra precision will be lost. Hex, octal, and binary numbers are converted to integers, so serializing those values after decoding them from a TOML document will be in their decimal representation. Example Usage The following is a brief overview of how to use this library. First, let's take a look at an example TOML file, as borrowed from the TOML homepage : # This is a TOML document. title = &quot;TOML Example&quot; [owner] name = &quot;Tom Preston-Werner&quot; dob = 1979-05-27T07:32:00-08:00 # First class dates [database] server = &quot;192.168.1.1&quot; ports = [ 8001, 8001, 8002 ] connection_max = 5000 enabled = true [servers] # Indentation (tabs and/or spaces) is allowed but not required [servers.alpha] ip = &quot;10.0.0.1&quot; dc = &quot;eqdc10&quot; [servers.beta] ip = &quot;10.0.0.2&quot; dc = &quot;eqdc10&quot; [clients] data = [ [&quot;gamma&quot;, &quot;delta&quot;], [1, 2] ] # Line breaks are OK when inside arrays hosts = [ &quot;alpha&quot;, &quot;omega&quot; ] Parsing iex&gt; input = &quot;&quot;&quot; [database] server = &quot;192.168.1.1&quot; &quot;&quot;&quot; ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode ( input ) ...&gt; { :ok , %{ database : %{ server : &quot;192.168.1.1&quot; } } } = Toml . decode ( input , keys : :atoms ) ...&gt; stream = File . stream! ( &quot;example.toml&quot; ) ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode_stream ( stream ) ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode_file ( &quot;example.toml&quot; ) ...&gt; invalid = &quot;&quot;&quot; [invalid] a = 1 b = 2 &quot;&quot;&quot; ...&gt; { :error , { :invalid_toml , reason } } = Toml . decode ( invalid ) ; IO . puts ( reason ) expected &#39; \\n &#39; , but got &#39;b&#39; in nofile on line 2 : a = 1 b = 2 ^ :ok Transforms Support for extending value conversions is provided by the Toml.Transform behavior. An example is shown below: Given the following TOML document: [servers.alpha] ip = &quot;192.168.1.1&quot; ports = [8080, 8081] [servers.beta] ip = &quot;192.168.1.2&quot; ports = [8082, 8083] And the following modules: defmodule Server do defstruct [ :name , :ip , :ports ] end defmodule IPStringToCharlist do use Toml.Transform def transform ( :ip , v ) when is_binary ( v ) do String . to_charlist ( v ) end def transform ( _k , v ) , do : v end defmodule CharlistToIP do use Toml.Transform def transform ( :ip , v ) when is_list ( v ) do case :inet . parse_ipv4_address ( v ) do { :ok , address } -&gt; address { :error , reason } -&gt; { :error , { :invalid_ip_address , reason } } end end def transform ( :ip , v ) , do : { :error , { :invalid_ip_address , v } } def transform ( _k , v ) , do : v end defmodule ServerMapToList do use Toml.Transform def transform ( :servers , v ) when is_map ( v ) do for { name , server } &lt;- v , do : struct ( Server , Map . put ( server , :name , name ) ) end def transform ( _k , v ) , do : v end You can convert the TOML document to a more strongly-typed version using the above transforms like so: iex&gt; transforms = [ IPStringToCharlist , CharlistToIP , ServerMapToList ] ...&gt; { :ok , result } = Toml . decode ( &quot;example.toml&quot; , keys : :atoms , transforms : transforms ) %{ servers : [ % Server { name : :alpha , ip : { 192 , 168 , 1 , 1 } } , ports : [ 8080 , 8081 ] | _ ] } The transforms given here are intended to show how they can be composed: they are applied in the order provided, and the document is transformed using a depth-first, bottom-up traversal. Put another way, you transform the leaves of the tree before the branches; as shown in the example above, this means the :ip key is converted to an address tuple before the :servers key is transformed into a list of Server structs. Using with Elixir Releases (1.9+) To use this library as a configuration provider in Elixir, use the following example of how one might use it in their release configuration, and tailor it to your needs: config_providers : [ { Toml.Provider , [ path : { :system , &quot;XDG_CONFIG_DIR&quot; , &quot;myapp.toml&quot; } , transforms : [ ... ] ] } ] See the &quot;Using as a Config Provider&quot; section for more info. Using with Distillery Like the above, use the following example as a guideline for how you use this in your own release configuration (i.e. in rel/config.exs ): release :myapp do # ...snip... set config_providers : [ { Toml.Provider , [ path : &quot;${XDG_CONFIG_DIR}/myapp.toml&quot; , transforms : [ ... ] ] } ] end Using as a Config Provider The usages described above will result in Toml.Provider being invoked during boot, at which point it will evaluate the given path and read the TOML file it finds. If one is not found, or is not accessible, the provider will raise an error, and the boot sequence will terminate unsuccessfully. If it succeeds, it persists settings in the file to the application environment (i.e. you access it via Application.get_env/2 ). You can pass the same options in the arguments list for Toml.Provider as you can to Toml.decode/2 , but :path is required, and :keys only supports :atoms and :atoms! values. The config provider expects a certain format to the TOML file, namely that keys at the root of the document correspond to applications which need to be configured. If it encounters keys at the root of the document which are not tables, they are ignored. # This is an example of something that would be ignored title = &quot;My config file&quot; # We're expecting something like this: [myapp] key = &quot;value&quot; # To use a bit of Phoenix config, you translate to TOML like so: [myapp.&quot;MyApp.Endpoint&quot;] cache_static_manifest = &quot;priv/static/cache_manifest.json&quot; [myapp.&quot;MyApp.Endpoint&quot;.http] port = &quot;4000&quot; [myapp.&quot;MyApp.Endpoint&quot;.force_ssl] hsts = true # Or logger.. [logger] level = &quot;info&quot; [logger.console] format = &quot;[$level] $message \\n&quot; Roadmap [x] Add benchmarking suite [x] Provide options for converting keys to atom, similar to Jason/Poison/etc. [ ] Optimize lexer to always send offsets to decoder, rather than only in some cases [ ] Try to find pathological TOML files to test License This project is licensed Apache 2.0, see the LICENSE file in this repo for details.","ref":"Toml.html","title":"Toml","type":"module"},{"doc":"Decode the given binary as TOML content Options You can pass the following options to configure the decoder behavior: :filename - pass a filename to use in error messages :keys - controls how keys in the document are decoded. Possible values are: :strings (default) - decodes keys as strings :atoms - converts keys to atoms with String.to_atom/1 :atoms! - converts keys to atoms with String.to_existing_atom/1 (key -&gt; term) - converts keys using the provided function :transforms - a list of custom transformations to apply to decoded TOML values, see Toml.Transform.transform/2 for details. Decoding keys to atoms The :atoms option uses the String.to_atom/1 call that can create atoms at runtime. Since the atoms are not garbage collected, this can pose a DoS attack vector when used on user-controlled data. It is recommended that if you either avoid converting to atoms, by using keys: :strings , or require known keys, by using the keys: :atoms! option, which will cause decoding to fail if the key is not an atom already in the atom table. Transformations You should rarely need custom datatype transformations, but in some cases it can be quite useful. In particular if you want to transform things like IP addresses from their string form to the Erlang address tuples used in most :inet APIs, a custom transform can ensure that all addresses are usable right away, and that validation of those addresses is done as part of decoding the document. Keep in mind that transforms add additional work to decoding, which may result in reduced performance, if you don't need the convenience, or the validation, deferring such conversions until the values are used may be a better approach, rather than incurring the overhead during decoding.","ref":"Toml.html#decode/2","title":"Toml.decode/2","type":"function"},{"doc":"Same as decode/1 , but returns the document directly, or raises Toml.Error if it fails.","ref":"Toml.html#decode!/2","title":"Toml.decode!/2","type":"function"},{"doc":"Decode the file at the given path as TOML Takes same options as decode/2","ref":"Toml.html#decode_file/2","title":"Toml.decode_file/2","type":"function"},{"doc":"Same as decode_file/1 , but returns the document directly, or raises Toml.Error if it fails.","ref":"Toml.html#decode_file!/2","title":"Toml.decode_file!/2","type":"function"},{"doc":"Decode the given stream as TOML. Takes same options as decode/2","ref":"Toml.html#decode_stream/2","title":"Toml.decode_stream/2","type":"function"},{"doc":"Same as decode_stream/1 , but returns the document directly, or raises Toml.Error if it fails.","ref":"Toml.html#decode_stream!/2","title":"Toml.decode_stream!/2","type":"function"},{"doc":"","ref":"Toml.html#t:error/0","title":"Toml.error/0","type":"type"},{"doc":"","ref":"Toml.html#t:key/0","title":"Toml.key/0","type":"type"},{"doc":"","ref":"Toml.html#t:opt/0","title":"Toml.opt/0","type":"type"},{"doc":"","ref":"Toml.html#t:opts/0","title":"Toml.opts/0","type":"type"},{"doc":"","ref":"Toml.html#t:reason/0","title":"Toml.reason/0","type":"type"},{"doc":"This module provides an implementation of both the Distilery and Elixir config provider behaviours, so that TOML files can be used for configuration in releases. Distillery Usage Add the following to your rel/config.exs release :myapp do # ...snip... set config_providers : [ { Toml.Provider , [ path : &quot;${XDG_CONFIG_DIR}/myapp.toml&quot; , transforms : [ ... ] ] } ] end Elixir Usage config_providers : [ { Toml.Provider , [ path : { :system , &quot;XDG_CONFIG_DIR&quot; , &quot;myapp.toml&quot; } , transforms : [ ... ] ] } ] This will result in Toml.Provider being invoked during boot, at which point it will evaluate the given path and read the TOML file it finds. If one is not found, or is not accessible, the provider will raise an error, and the boot sequence will terminate unsuccessfully. If it succeeds, it persists settings in the file to the application environment (i.e. you access it via Application.get_env/2 ). The config provider expects a certain format to the TOML file, namely that keys at the root of the document are tables which correspond to applications which need to be configured. If it encounters keys at the root of the document which are not tables, they are ignored. Options The same options that Toml.parse/2 accepts are able to be provided to Toml.Provider , but there are two main differences: :path (required) - sets the path to the TOML file to load config from :keys - defaults to :atoms , but can be set to :atoms! if desired, all other key types are ignored, as it results in an invalid config structure","ref":"Toml.Provider.html","title":"Toml.Provider","type":"module"},{"doc":"","ref":"Toml.Provider.html#expand_path/1","title":"Toml.Provider.expand_path/1","type":"function"},{"doc":"Defines the behavior for custom transformations of decoded TOML values. See the documentation for transform/2 for more details.","ref":"Toml.Transform.html","title":"Toml.Transform","type":"behaviour"},{"doc":"This function is invoked for every key/value pair in the document, in a depth-first, bottom-up, traversal of the document. The transformer must return one of two values: {:error, term} - an error occurred in the transformer, and decoding should fail term - replace the value for the given key with the value returned from the transformer in the final document Example An example transformation would be the conversion of tables of a certain shape to a known struct value. The struct: defmodule Server do defstruct [ :name , :ip , :ports ] end TOML which contains a table of this shape: [ servers . alpha ] ip = &quot;192.168.1.1&quot; ports = [ 8080 , 8081 ] [ servers . beta ] ip = &quot;192.168.1.2&quot; ports = [ 8082 , 8083 ] And finally, the transforer implementation: defmodule ServerTransform do use Toml.Transform # Transform IP strings to Erlang address tuples def transform ( :ip , ip ) when is_binary ( ip ) do case :inet . parse_ipv4_address ( String . to_charlist ( ip ) ) do { :ok , result } -&gt; result { :error , reason } -&gt; { :error , { :invalid_ip , ip , reason } } end end # Non-binary values for IP addresses should return an error def transform ( :ip , ip ) , do : { :error , { :invalid_ip , ip , :expected_string } } # Transform the server objects to Server structs def transform ( :servers , servers ) when is_map ( servers ) do for { name , server } &lt;- servers do struct ( Server , Map . put ( server , :name , name ) ) end end # Non-map values for servers should return an error def transform ( :servers , s ) , do : { :error , { :invalid_server , s } } # Ignore all other values def transform ( _key , v ) , do : v end Assuming we decode with the following options: Toml . decode! ( content , keys : :atoms , transforms : [ ServerTransform ] ) The result would be: %{ servers : [ % Server { name : :alpha , ip : { 192 , 168 , 1 , 1 } , ports : [ 8080 , 8081 ] } , % Server { name : :beta , ip : { 192 , 168 , 1 , 2 } , ports : [ 8082 , 8083 ] } ] }","ref":"Toml.Transform.html#c:transform/2","title":"Toml.Transform.transform/2","type":"callback"},{"doc":"","ref":"Toml.Transform.html#t:key/0","title":"Toml.Transform.key/0","type":"type"},{"doc":"","ref":"Toml.Transform.html#t:keypath/0","title":"Toml.Transform.keypath/0","type":"type"},{"doc":"","ref":"Toml.Transform.html#t:t/0","title":"Toml.Transform.t/0","type":"type"},{"doc":"","ref":"Toml.Transform.html#t:value/0","title":"Toml.Transform.value/0","type":"type"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"License","type":"extras"},{"doc":"This is a TOML library for Elixir projects. It is compliant with version 1.0 of the official TOML specification . You can find a brief overview of the feature set below, but you are encouraged to read the full spec at the link above (it is short and easy to read!).","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Decode from string, file, or stream Fully compliant with the latest version of the TOML spec Is tested against toml-test , a test suite for spec-compliant TOML encoders/decoders, used by implementations in multiple languages. The test suite has been integrated into this project to be run under Mix so that we get better error information and so it can run as part of the test suite. Decoder produces a map with values using the appropriate Elixir data types for representation Supports extension via value transformers (see Toml.Transform docs for details) Supports use as a configuration provider in Distillery 2.x+ (use TOML files for configuration!) Decoder is written by hand to take advantage of various optimizations. Library passes Dialyzer checks","ref":"readme.html#features","title":"Overview - Features","type":"extras"},{"doc":"I compared toml to four other libraries: toml_elixir tomlex jerry etoml Of these four, none correctly implement the 0.5.0 specification. Either they are targeting older versions of the spec (in etoml , it is built against pre-0.1), are not fully implemented (i.e. don't support all features), or have bugs which prevent them from properly parsing a 0.5.0 example file (the test/fixtures/example.toml file in this repository). If you are looking for a TOML library, at present toml is the only one which full implements the spec and correctly decodes example.toml .","ref":"readme.html#comparison-to-other-libraries","title":"Overview - Comparison To Other Libraries","type":"extras"},{"doc":"This library is available on Hex as :toml , and can be added to your deps like so: def deps do [ { :toml , &quot;~&gt; 0.7&quot; } ] end NOTE: You can determine the latest version on Hex by running mix hex.info toml .","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"In case you are curious how TOML types are translated to Elixir types, the following table provides the conversions. NOTE: The various possible representations of each type, such as hex/octal/binary integers, quoted/literal strings, etc., are considered to be the same base type (e.g. integer and string respectively in the examples given). TOML Elixir String String.t (binary) Integer integer inf :infinity +inf :infinity -inf :negative_infinity nan :nan +nan :nan -nan :negative_nan Boolean boolean Offset Date-Time DateTime.t Local Date-Time NaiveDateTime.t Local Date Date.t Local Time Time.t Array list Table map Table Array list(map)","ref":"readme.html#type-conversions","title":"Overview - Type Conversions","type":"extras"},{"doc":"Certain features of TOML have implementation-specific behavior: -inf , inf , and +inf are all valid infinity values in TOML. In Erlang/Elixir, these don't have exact representations. Instead, by convention, :infinity is used for positive infinity, as atoms are always larger than integers when using comparison operators, so :infinity &gt; &lt;any integer&gt; will always be true. However, negative infinity cannot be represented, as numbers are always considered smaller than every other type in term comparisons. Instead, we represent it with :negative_infinity , so that the type information is not lost, but you must be careful to deal with it specifically in comparisons/sorting/etc. -nan , nan , and +nan are all valid NaN (not a number) values in TOML. In Erlang/Elixir, NaN is traditionally represented with :nan , but there is no representation for negative NaN, and no API actually produces :nan , instead invalid numbers typically raise errors, in the typical spirit of &quot;let it crash&quot; in the face of errors. For purposes of preserving type information though, we use the :nan convention, and :negative_nan for -NaN. You will need to take care to deal with these values manually if the values need to be preserved. The maximum precision of times in the various time types is microseconds (i.e. precision to six decimal places), if you provide higher precision values (i.e. nanoseconds), the extra precision will be lost. Hex, octal, and binary numbers are converted to integers, so serializing those values after decoding them from a TOML document will be in their decimal representation.","ref":"readme.html#implementation-specific-behaviors","title":"Overview - Implementation-specific Behaviors","type":"extras"},{"doc":"The following is a brief overview of how to use this library. First, let's take a look at an example TOML file, as borrowed from the TOML homepage : # This is a TOML document. title = &quot;TOML Example&quot; [owner] name = &quot;Tom Preston-Werner&quot; dob = 1979-05-27T07:32:00-08:00 # First class dates [database] server = &quot;192.168.1.1&quot; ports = [ 8001, 8001, 8002 ] connection_max = 5000 enabled = true [servers] # Indentation (tabs and/or spaces) is allowed but not required [servers.alpha] ip = &quot;10.0.0.1&quot; dc = &quot;eqdc10&quot; [servers.beta] ip = &quot;10.0.0.2&quot; dc = &quot;eqdc10&quot; [clients] data = [ [&quot;gamma&quot;, &quot;delta&quot;], [1, 2] ] # Line breaks are OK when inside arrays hosts = [ &quot;alpha&quot;, &quot;omega&quot; ] Parsing iex&gt; input = &quot;&quot;&quot; [database] server = &quot;192.168.1.1&quot; &quot;&quot;&quot; ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode ( input ) ...&gt; { :ok , %{ database : %{ server : &quot;192.168.1.1&quot; } } } = Toml . decode ( input , keys : :atoms ) ...&gt; stream = File . stream! ( &quot;example.toml&quot; ) ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode_stream ( stream ) ...&gt; { :ok , %{ &quot;database&quot; =&gt; %{ &quot;server&quot; =&gt; &quot;192.168.1.1&quot; } } } = Toml . decode_file ( &quot;example.toml&quot; ) ...&gt; invalid = &quot;&quot;&quot; [invalid] a = 1 b = 2 &quot;&quot;&quot; ...&gt; { :error , { :invalid_toml , reason } } = Toml . decode ( invalid ) ; IO . puts ( reason ) expected &#39; \\n &#39; , but got &#39;b&#39; in nofile on line 2 : a = 1 b = 2 ^ :ok Transforms Support for extending value conversions is provided by the Toml.Transform behavior. An example is shown below: Given the following TOML document: [servers.alpha] ip = &quot;192.168.1.1&quot; ports = [8080, 8081] [servers.beta] ip = &quot;192.168.1.2&quot; ports = [8082, 8083] And the following modules: defmodule Server do defstruct [ :name , :ip , :ports ] end defmodule IPStringToCharlist do use Toml.Transform def transform ( :ip , v ) when is_binary ( v ) do String . to_charlist ( v ) end def transform ( _k , v ) , do : v end defmodule CharlistToIP do use Toml.Transform def transform ( :ip , v ) when is_list ( v ) do case :inet . parse_ipv4_address ( v ) do { :ok , address } -&gt; address { :error , reason } -&gt; { :error , { :invalid_ip_address , reason } } end end def transform ( :ip , v ) , do : { :error , { :invalid_ip_address , v } } def transform ( _k , v ) , do : v end defmodule ServerMapToList do use Toml.Transform def transform ( :servers , v ) when is_map ( v ) do for { name , server } &lt;- v , do : struct ( Server , Map . put ( server , :name , name ) ) end def transform ( _k , v ) , do : v end You can convert the TOML document to a more strongly-typed version using the above transforms like so: iex&gt; transforms = [ IPStringToCharlist , CharlistToIP , ServerMapToList ] ...&gt; { :ok , result } = Toml . decode ( &quot;example.toml&quot; , keys : :atoms , transforms : transforms ) %{ servers : [ % Server { name : :alpha , ip : { 192 , 168 , 1 , 1 } } , ports : [ 8080 , 8081 ] | _ ] } The transforms given here are intended to show how they can be composed: they are applied in the order provided, and the document is transformed using a depth-first, bottom-up traversal. Put another way, you transform the leaves of the tree before the branches; as shown in the example above, this means the :ip key is converted to an address tuple before the :servers key is transformed into a list of Server structs.","ref":"readme.html#example-usage","title":"Overview - Example Usage","type":"extras"},{"doc":"To use this library as a configuration provider in Elixir, use the following example of how one might use it in their release configuration, and tailor it to your needs: config_providers : [ { Toml.Provider , [ path : { :system , &quot;XDG_CONFIG_DIR&quot; , &quot;myapp.toml&quot; } , transforms : [ ... ] ] } ] See the &quot;Using as a Config Provider&quot; section for more info.","ref":"readme.html#using-with-elixir-releases-1-9","title":"Overview - Using with Elixir Releases (1.9+)","type":"extras"},{"doc":"Like the above, use the following example as a guideline for how you use this in your own release configuration (i.e. in rel/config.exs ): release :myapp do # ...snip... set config_providers : [ { Toml.Provider , [ path : &quot;${XDG_CONFIG_DIR}/myapp.toml&quot; , transforms : [ ... ] ] } ] end","ref":"readme.html#using-with-distillery","title":"Overview - Using with Distillery","type":"extras"},{"doc":"The usages described above will result in Toml.Provider being invoked during boot, at which point it will evaluate the given path and read the TOML file it finds. If one is not found, or is not accessible, the provider will raise an error, and the boot sequence will terminate unsuccessfully. If it succeeds, it persists settings in the file to the application environment (i.e. you access it via Application.get_env/2 ). You can pass the same options in the arguments list for Toml.Provider as you can to Toml.decode/2 , but :path is required, and :keys only supports :atoms and :atoms! values. The config provider expects a certain format to the TOML file, namely that keys at the root of the document correspond to applications which need to be configured. If it encounters keys at the root of the document which are not tables, they are ignored. # This is an example of something that would be ignored title = &quot;My config file&quot; # We're expecting something like this: [myapp] key = &quot;value&quot; # To use a bit of Phoenix config, you translate to TOML like so: [myapp.&quot;MyApp.Endpoint&quot;] cache_static_manifest = &quot;priv/static/cache_manifest.json&quot; [myapp.&quot;MyApp.Endpoint&quot;.http] port = &quot;4000&quot; [myapp.&quot;MyApp.Endpoint&quot;.force_ssl] hsts = true # Or logger.. [logger] level = &quot;info&quot; [logger.console] format = &quot;[$level] $message \\n&quot;","ref":"readme.html#using-as-a-config-provider","title":"Overview - Using as a Config Provider","type":"extras"},{"doc":"[x] Add benchmarking suite [x] Provide options for converting keys to atom, similar to Jason/Poison/etc. [ ] Optimize lexer to always send offsets to decoder, rather than only in some cases [ ] Try to find pathological TOML files to test","ref":"readme.html#roadmap","title":"Overview - Roadmap","type":"extras"},{"doc":"This project is licensed Apache 2.0, see the LICENSE file in this repo for details.","ref":"readme.html#license","title":"Overview - License","type":"extras"}]